@@ -1,8 +1,9 @@
 (* camlp5r *)
 (* pa_lefteval.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "q_MLast.cmo";
+#load "pa_macro.cmo";
 
 value not_impl name x =
   let desc =
@@ -54,7 +55,7 @@ value rec may_side_effect =
 value rec may_be_side_effect_victim =
   fun
   [ <:expr< $lid:_$ . $_$ >> -> True
-  | <:expr< $uid:_$ . $e$ >> -> may_be_side_effect_victim e
+  | <:expr< $longid:_$ . ( $e$ ) >> -> may_be_side_effect_victim e
   | _ -> False ]
 ;
 
@@ -142,7 +143,7 @@ value map_vala f x = <:vala< f (Pcaml.unvala x) >>;
 value class_infos f ci =
   {MLast.ciLoc = ci.MLast.ciLoc; MLast.ciVir = ci.MLast.ciVir;
    MLast.ciPrm = ci.MLast.ciPrm; MLast.ciNam = ci.MLast.ciNam;
-   MLast.ciExp = f ci.MLast.ciExp}
+   MLast.ciExp = f ci.MLast.ciExp; MLast.ciAttributes = ci.MLast.ciAttributes}
 ;
 
 value rec expr x =
@@ -156,8 +157,8 @@ value rec expr x =
       <:expr< try $expr e$ with [ $list:List.map match_assoc pwel$ ] >>
   | <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
       <:expr< let $flag:rf$ $list:List.map let_binding pel$ in $expr e$ >>
-  | <:expr< let module $uid:s$ = $me$ in $e$ >> ->
-      <:expr< let module $uid:s$ = $module_expr me$ in $expr e$ >>
+  | <:expr< let module $uidopt:s$ = $me$ in $e$ >> ->
+      <:expr< let module $uidopt:s$ = $module_expr me$ in $expr e$ >>
   | <:expr< if $e1$ then $e2$ else $e3$ >> ->
       <:expr< if $expr e1$ then $expr e2$ else $expr e3$ >>
   | <:expr< while $e$ do { $list:el$ } >> ->
@@ -176,16 +177,17 @@ value rec expr x =
   | <:expr< $e1$.[$e2$] >> -> <:expr< $expr e1$.[$expr e2$] >>
   | <:expr< $_$ . $_$ >> | <:expr< $uid:_$ >> | <:expr< $lid:_$ >> |
     <:expr< $str:_$ >> | <:expr< $chr:_$ >> | <:expr< $int:_$ >> |
-    <:expr< $flo:_$ >> | <:expr< new $list:_$ >> ->
-      x
+    <:expr< $flo:_$ >>
+  | <:expr< new $longid:_$ . $lid:_$ >> | <:expr< new $lid:_$ >>
+     -> x
   | x -> not_impl "expr" x ]
-and let_binding (p, e) = (p, expr e)
+and let_binding (p, e, attrs) = (p, expr e, attrs)
 and match_assoc (p, eo, e) = (p, map_vala (map_option expr) eo, expr e)
 and module_expr x =
   let loc = MLast.loc_of_module_expr x in
   match x with
-  [ <:module_expr< functor ($uid:s$ : $mt$) -> $me$ >> ->
-      <:module_expr< functor ($uid:s$ : $mt$) -> $module_expr me$ >>
+  [ <:module_expr< functor $_fp:arg$ -> $me$ >> ->
+      <:module_expr< functor $_fp:arg$ -> $module_expr me$ >>
   | <:module_expr< ($me$ : $mt$) >> ->
       <:module_expr< ($module_expr me$ : $mt$) >>
   | <:module_expr< struct $list:sil$ end >> ->
@@ -204,7 +206,8 @@ and str_item x =
       <:str_item< class $list:List.map (class_infos class_expr) ce$ >>
   | <:str_item< $exp:e$ >> -> <:str_item< $exp:expr e$ >>
   | <:str_item< open $_$ >> | <:str_item< type $list:_$ >> |
-    <:str_item< exception $uid:_$ of $list:_$ = $_$ >> |
+    <:str_item< exception $uid:_$ of $list:_$ >> |
+    <:str_item< exception $uid:_$ = $longid:_$ >> |
     <:str_item< module type $_$ = $_$ >> |
     <:str_item< # $lid:_$ $opt:_$ >> ->
       x
@@ -225,9 +228,8 @@ and class_str_item x =
   | x -> not_impl "class_str_item" x ]
 ;
 
-value parse_implem = Pcaml.parse_implem.val;
-value parse_implem_with_left_eval strm =
-  let (r, b) = parse_implem strm in
+value left_eval_transform (r,b) =
   (List.map (fun (si, loc) -> (str_item si, loc)) r, b)
 ;
-Pcaml.parse_implem.val := parse_implem_with_left_eval;
+
+Pcaml.(set_ast_transform transduce_implem left_eval_transform);
