@@ -263,7 +263,7 @@
 (define (lexer_using kwt (values con prm))
  (match con
   ((or "CHAR" "DOT" "EOI" "INT" "INT_l" "INT_L" "INT_n" "FLOAT" "LIDENT" "NL"
-    "QUOT" "SPACEDOT" "STRING" "UIDENT")
+    "QUOT" "SPACEDOT" "STRING" "RAWSTRING" "UIDENT")
    ())
   ((or "ANTIQUOT" "ANTIQUOT_LOC") ())
   ("" (try (begin (Hashtbl.find kwt prm) ()) (Not_found (Hashtbl.add kwt prm prm))))
@@ -1462,6 +1462,10 @@ EXTEND
       | s = V FLOAT -> (Sfloat loc s)
       | s = V CHAR -> (Schar loc s)
       | s = V STRING -> (Sstring loc s)
+      | s = RAWSTRING ->
+         (let* (((values _ s) (Asttools.split_rawstring s)))
+                 (Sstring loc (Ploc.VaVal s))
+         )
       | s = SPACEDOT -> (Slid loc ".")
       | s = QUOT ->
           (let* ((i (String.index s ':'))
