@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pr_scheme.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #directory ".";
 #load "pa_macro.cmo";
@@ -20,6 +20,7 @@ do {
   Eprinter.clear pr_ctyp;
   Eprinter.clear pr_str_item;
   Eprinter.clear pr_sig_item;
+  Eprinter.clear pr_longident;
   Eprinter.clear pr_module_expr;
   Eprinter.clear pr_module_type;
   Eprinter.clear pr_class_sig_item;
@@ -50,19 +51,11 @@ value to_be_renamed = ["cond"; "sum"];
 
 value rename_id s = if List.mem s to_be_renamed then s ^ "#" else s;
 
-IFDEF OCAML_VERSION <= OCAML_1_07 THEN
-  value with_ind_bef = Pprintf.with_ind_bef;
-  value with_ind_bef_aft = Pprintf.with_ind_bef_aft;
-  value with_bef = Pprintf.with_bef;
-  value with_bef_aft = Pprintf.with_bef_aft;
-  value with_aft = Pprintf.with_aft;
-END;
-
-value rec longident pc sl =
+value rec id_list pc sl =
   match sl with
   [ [] -> pprintf pc ""
   | [s] -> pprintf pc "%s" s
-  | [s :: sl] -> pprintf pc "%s.%p" s longident sl ]
+  | [s :: sl] -> pprintf pc "%s.%p" s id_list sl ]
 ;
 
 (*
@@ -74,6 +67,7 @@ value patt = Eprinter.apply pr_patt;
 value ctyp = Eprinter.apply pr_ctyp;
 value str_item = Eprinter.apply pr_str_item;
 value sig_item = Eprinter.apply pr_sig_item;
+value longident = Eprinter.apply pr_longident;
 value module_expr = Eprinter.apply pr_module_expr;
 value module_type = Eprinter.apply pr_module_type;
 value class_str_item = Eprinter.apply pr_class_str_item;
@@ -111,7 +105,7 @@ pr_expr_fun_args.val :=
 
 value has_cons_with_params vdl =
   List.exists
-    (fun (_, _, tl, rto) ->
+    (fun (_, _, _, tl, rto, _) ->
        match tl with
        [ <:vala< [] >> -> False
        | _ -> True ])
@@ -138,13 +132,17 @@ value braceless pc b =
    aft = sprintf ">}%s" pc.aft}
 ;
 
-value type_param pc (tv, vari) =
+value type_param pc (tv, (vari, inj)) =
   let tv = Pcaml.unvala tv in
-  pprintf pc "%s%s"
+  pprintf pc "%s%s%s"
     (match vari with
      [ Some True -> "+"
      | Some False -> "-"
      | None -> "" ])
+    (match inj with
+       [ True -> "!"
+       | False -> ""
+       ])
     (match tv with
      [ Some v -> "'" ^ v
      | None -> "_" ])
@@ -177,7 +175,7 @@ value exception_decl pc (c, tl) =
      List.map (fun t -> (fun pc -> ctyp pc t, "")) tl]
 ;
 
-value value_binding b pc (p, e) =
+value value_binding b pc (p, e, _) =
   let (pl, e) = expr_fun_args e in
   horiz_vertic
     (fun () ->
@@ -216,7 +214,7 @@ value value_binding b pc (p, e) =
 value value_binding_list pc (rf, pel) =
   let b = if rf then "definerec" else "define" in
   match pel with
-  [ [(p, e)] -> value_binding b pc (p, e)
+  [ [((_, _, _) as x)] -> value_binding b pc x
   | _ ->
       horiz_vertic
         (fun () ->
@@ -225,7 +223,7 @@ value value_binding_list pc (rf, pel) =
            pprintf pc "(%s*@;<1 1>%p)" b (vlist (value_binding "")) pel) ]
 ;
 
-value let_binding pc (p, e) =
+value let_binding pc (p, e,_) =
   let (pl, e) = expr_fun_args e in
   plistf 0 (paren pc "")
     [(fun pc ->
@@ -261,7 +259,8 @@ value match_assoc pc (p, we, e) =
   plistf 0 (paren pc "") list
 ;
 
-value constr_decl pc (_, c, tl, rto) =
+value constr_decl pc (_, c, _, tl, rto, alg_attrs) = do {
+  assert (alg_attrs = <:vala< [] >>) ;
   let c = Pcaml.unvala c in
   let tl = Pcaml.unvala tl in
   match tl with
@@ -270,6 +269,7 @@ value constr_decl pc (_, c, tl, rto) =
       plistf 0 (paren pc "")
         [(fun pc -> pprintf pc "%s" c, "") ::
          List.map (fun t -> (fun pc -> ctyp pc t, "")) tl] ]
+}
 ;
 
 value poly_variant_decl pc =
@@ -294,11 +294,13 @@ value poly_variant_decl pc =
     END ]
 ;
 
-value label_decl pc (_, l, m, t) =
+value label_decl pc (_, l, m, t, attrs) = do {
+  assert(attrs = Ploc.VaVal []) ;
   let list = [(fun pc -> ctyp pc t, "")] in
   plistf 0 (paren pc "")
     [(fun pc -> pprintf pc "%s" l, "") ::
      if m then [(fun pc -> pprintf pc "mutable", "") :: list] else list]
+}
 ;
 
 value module_type_decl pc (s, mt) =
@@ -312,6 +314,12 @@ value field_expr pc (l, e) =
     [(fun pc -> pprintf pc "%s" l, ""); (fun pc -> expr pc e, "")]
 ;
 
+value longident_lident pc = fun [
+  (None, id) -> pprintf pc "%s" (Pcaml.unvala id)
+| (Some li, id) -> pprintf pc "%p.%s" longident (Pcaml.unvala li) (Pcaml.unvala id)
+]
+;
+
 value string pc s = pprintf pc "\"%s\"" s;
 
 value int_repr s =
@@ -332,13 +340,13 @@ value int_repr s =
 
 value with_constraint pc =
   fun
-  [ <:with_constr< type $sl$ $list:tvl$ = $flag:pf$ $t$ >> ->
+  [ <:with_constr< type $lilongid:sl$ $list:tvl$ = $flag:pf$ $t$ >> ->
       pprintf pc "(type%s %p@;<1 1>%p)" (if pf then "private" else "")
         (fun pc ->
            fun
-           [ [] -> longident pc sl
+           [ [] -> longident_lident pc sl
            | tvl ->
-               pprintf pc "(%p %p)" longident sl
+               pprintf pc "(%p %p)" longident_lident sl
                  (hlist type_param) tvl ])
         tvl ctyp t
   | wc -> not_impl "with_constraint" pc wc ]
@@ -567,10 +575,8 @@ EXTEND_PRINTER
           plistb curr 0 (paren pc "==") [(t1, ""); (t2, "")]
       | <:ctyp< $t1$ as $t2$ >> ->
           plistb curr 0 (paren pc "as") [(t1, ""); (t2, "")]
-      | <:ctyp< $t1$ . $t2$ >> ->
-          sprintf "%s.%s"
-            (curr {(pc) with aft = ""} t1)
-            (curr {(pc) with bef = ""} t2)
+      | <:ctyp< $longid:me1$ . $lid:lid$ >> ->
+          pprintf pc "%p.%s" longident me1 lid
       | <:ctyp< < $list:fl$ $flag:v$ > >> ->
           let b = if v then "objectvar" else "object" in
           if fl = [] then sprintf "%s(%s)%s" pc.bef b pc.aft
@@ -585,14 +591,12 @@ EXTEND_PRINTER
              (fun pc -> curr pc t, "")]
       | <:ctyp< $lid:s$ >> ->
           sprintf "%s%s%s" pc.bef (rename_id s) pc.aft
-      | <:ctyp< $uid:s$ >> ->
-          sprintf "%s%s%s" pc.bef s pc.aft
       | <:ctyp< ' $s$ >> ->
           sprintf "%s'%s%s" pc.bef s pc.aft
       | <:ctyp< _ >> ->
           sprintf "%s_%s" pc.bef pc.aft
-      | <:ctyp< # $list:sl$ >> ->
-          pprintf pc "(# %p)" longident sl
+      | <:ctyp< # $lilongid:lili$ >> ->
+          pprintf pc "(# %p)" longident_lident lili
       | <:ctyp< ! $list:pl$ . $t$ >> ->
           pprintf pc "(! (%p)@;<1 1>%p)" (hlist type_var) pl ctyp t
       | x ->
@@ -666,10 +670,10 @@ EXTEND_PRINTER
                sprintf "%s\n%s" s1 s2)
       | <:expr< let $p1$ = $e1$ in $e2$ >> ->
           let (pel, e) =
-            loop [(p1, e1)] e2 where rec loop pel =
+            loop [(p1, e1, <:vala< [] >>)] e2 where rec loop pel =
               fun
               [ <:expr< let $p1$ = $e1$ in $e2$ >> ->
-                  loop [(p1, e1) :: pel] e2
+                  loop [(p1, e1, <:vala< [] >>) :: pel] e2
               | e -> (List.rev pel, e) ]
           in
           let b =
@@ -681,11 +685,16 @@ EXTEND_PRINTER
       | <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
           let b = if rf then "letrec" else "let" in
           let_binding_list pc (b, pel, e)
-      | <:expr< let module $uid:s$ = $me$ in $e$ >> ->
+      | <:expr< let module $uidopt:s$ = $me$ in $e$ >> ->
+        let s = match s with [ None -> "_" | Some uid -> Pcaml.unvala uid ] in
           plistbf 0 (paren pc "letmodule")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> module_expr pc me, "");
              (fun pc -> curr pc e, "")]
+      | <:expr< let open $uid:s$ in $e$ >> ->
+          plistbf 0 (paren pc "letopen")
+            [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
+             (fun pc -> curr pc e, "")]
       | <:expr< if $e1$ then $e2$ else () >> ->
           plistb curr 0 (paren pc "if") [(e1, ""); (e2, "")]
       | <:expr< if $e1$ then $e2$ else $e3$ >> ->
@@ -814,10 +823,10 @@ EXTEND_PRINTER
           plistf 0 (paren pc "")
             [(fun pc -> sprintf "%slazy%s" pc.bef pc.aft, "");
              (fun pc -> curr pc x, "")]
-      | <:expr< new $list:x$ >> ->
+      | <:expr< new $lilongid:lili$ >> ->
           plistf 0 (paren pc "")
             [(fun pc -> sprintf "%snew%s" pc.bef pc.aft, "");
-             (fun pc -> longident pc x, "")]
+             (fun pc -> longident_lident pc lili, "")]
 (*
       | <:expr< $lid:s$ $e1$ $e2$ >>
         when List.mem s assoc_right_parsed_op_list ->
@@ -867,10 +876,17 @@ EXTEND_PRINTER
           sprintf "%s%s.(%s)%s" pc.bef
             (curr {(pc) with bef = ""; aft = ""} e1)
             (curr {(pc) with bef = ""; aft = ""} e2) pc.aft
-      | <:expr< $e1$ . $e2$ >> ->
+      | <:expr< $longid:li$ >> -> sprintf "%s" (longident {(pc) with aft = ""} li)
+      | <:expr< $longid:li$ . ( $e$ ) >> ->
+          plistbf 0 (paren pc "letopen")
+            [(fun pc -> sprintf "%s%s%s" pc.bef (longident pc li)  pc.aft, "");
+             (fun pc -> curr pc e, "")]
+
+      | <:expr< $e$ . $lilongid:lili$ >> ->
            sprintf "%s.%s"
-             (curr {(pc) with aft = ""} e1)
-             (curr {(pc) with bef = ""} e2)
+             (curr {(pc) with aft = ""} e)
+             (longident_lident {(pc) with bef = ""} lili)
+
       | <:expr< $int:s$ >> ->
           sprintf "%s%s%s" pc.bef (int_repr s) pc.aft
       | <:expr< $int32:s$ >> ->
@@ -1024,9 +1040,9 @@ EXTEND_PRINTER
           plistbf 0 (paren pc "~")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> curr pc p, "")]
-      | <:patt< $p1$ . $p2$ >> ->
+      | <:patt< $longid:p1$ . $p2$ >> ->
            sprintf "%s.%s"
-             (curr {(pc) with aft = ""} p1)
+             (longident {(pc) with aft = ""} p1)
              (curr {(pc) with bef = ""} p2)
       | <:patt< $lid:s$ >> ->
           sprintf "%s%s%s" pc.bef (rename_id s) pc.aft
@@ -1052,8 +1068,8 @@ EXTEND_PRINTER
           sprintf "%s(` %s)%s" pc.bef s pc.aft
       | <:patt< _ >> ->
           sprintf "%s_%s" pc.bef pc.aft
-      | <:patt< # $list:sl$ >> ->
-          pprintf pc "(# %p)" longident sl
+      | <:patt< # $lilongid:lili$ >> ->
+          pprintf pc "(# %p)" longident_lident lili
       | x ->
           not_impl "patt" pc x ] ]
   ;
@@ -1063,24 +1079,28 @@ EXTEND_PRINTER
           class_decl_list pc cdl
       | <:str_item< class type $list:ctdl$ >> ->
           class_type_decl_list pc ctdl
-      | <:str_item< open $i$ >> ->
-          plistb longident 0 (paren pc "open") [(i, "")]
+      | <:str_item< open $me$ >> ->
+          plistb module_expr 0 (paren pc "open") [(me, "")]
       | <:str_item< include $me$ >> ->
           plistb module_expr 0 (paren pc "include") [(me, "")]
       | <:str_item< type $list:tdl$ >> ->
           type_decl_list pc tdl
       | <:str_item< exception $uid:c$ of $list:tl$ >> ->
           exception_decl pc (c, tl)
-      | <:str_item< exception $uid:c$ of $list:_$ = $id$ >> ->
+      | <:str_item< exception $uid:c$ = $longid:li$ >> ->
           plistbf 0 (paren pc "exceptionrebind")
             [(fun pc -> sprintf "%s%s%s" pc.bef c pc.aft, "");
-             (fun pc -> longident pc id, "")]
+             (fun pc -> longident pc li, "")]
       | <:str_item< value $flag:rf$ $list:pel$ >> ->
           value_binding_list pc (rf, pel)
       | <:str_item< module $uid:s$ = $me$ >> ->
           plistbf 0 (paren pc "module")
             [(fun pc -> sprintf "%s%s%s" pc.bef s pc.aft, "");
              (fun pc -> module_expr pc me, "")]
+      | <:str_item< module _ = $me$ >> ->
+          plistbf 0 (paren pc "module")
+            [(fun pc -> sprintf "%s_%s" pc.bef pc.aft, "");
+             (fun pc -> module_expr pc me, "")]
       | <:str_item< module type $s$ = $mt$ >> ->
           module_type_decl pc (s, mt)
       | <:str_item< external $lid:i$ : $t$ = $list:pd$ >> ->
@@ -1126,7 +1146,7 @@ EXTEND_PRINTER
              (fun pc -> module_type pc mt, "")]
       | <:sig_item< module type $s$ = $mt$ >> ->
           module_type_decl pc (s, mt)
-      | <:sig_item< open $i$ >> ->
+      | <:sig_item< open $longid:i$ >> ->
           plistb longident 0 (paren pc "open") [(i, "")]
       | <:sig_item< type $list:tdl$ >> ->
           type_decl_list pc tdl
@@ -1143,6 +1163,20 @@ EXTEND_PRINTER
       | x ->
           not_impl "sig_item" pc x ] ]
   ;
+  pr_longident:
+    [ "dot"
+      [ <:extended_longident< $longid:x$ . $uid:uid$ >> ->
+          pprintf pc "%p.%s" curr x uid
+      | <:extended_longident< $longid:x$ ( $longid:y$ ) >> ->
+          pprintf pc "%p(%p)" longident x longident y
+      | <:extended_longident< $uid:s$ >> ->
+          pprintf pc "%s" s
+      ]
+    | "bottom" [
+        z -> pprintf pc "[INTERNAL ERROR(pr_longident): unexpected longident]"
+      ]
+    ]
+  ;
   pr_module_expr:
     [ "top"
       [ <:module_expr< functor ($uid:i$ : $mt$) -> $me$ >> ->
@@ -1201,12 +1235,13 @@ EXTEND_PRINTER
                sprintf "%s\n%s" s1 s2)
       | <:module_type< $mt$ with $list:wcl$ >> ->
           pprintf pc "(with %p@;<1 1>%p)" curr mt (hlist with_constraint) wcl
-      | <:module_type< $me1$ . $me2$ >> ->
-           sprintf "%s.%s"
-             (curr {(pc) with aft = ""} me1)
-             (curr {(pc) with bef = ""} me2)
-      | <:module_type< $uid:s$ >> ->
-          sprintf "%s%s%s" pc.bef s pc.aft
+
+      | <:module_type< $longid:li$ . $lid:s$ >> ->
+          pprintf pc "%p.%s" longident li s
+      | <:module_type< $longid:li$ >> ->
+          pprintf pc "%p" longident li
+      | <:module_type< $lid:s$ >> ->
+          pprintf pc "%s" s
       | x ->
           not_impl "module_type" pc x ] ]
   ;
@@ -1398,10 +1433,12 @@ EXTEND_PRINTER
           plistf 0 (paren pc "")
             [(fun pc -> curr pc ce, "") ::
              List.map (fun e -> (fun pc -> expr pc e, "")) el]
-      | <:class_expr< $list:sl$ >> ->
-          longident pc sl
-      | <:class_expr< [ $list:ctcl$ ] $list:sl$ >> ->
-          not_impl "CeCon" pc sl
+      | <:class_expr< $lilongid:lili$ >> ->
+          longident_lident pc lili
+      | <:class_expr< [ $list:ctcl$ ] $longid:li$ . $lid:id$ >> ->
+          not_impl "CeCon" pc ()
+      | <:class_expr< [ $list:ctcl$ ] $lid:id$ >> ->
+          not_impl "CeCon" pc ()
       | x ->
           not_impl "class_expr" pc x ] ]
   ;
@@ -1447,8 +1484,9 @@ module Buff =
   struct
     value buff = ref (string_create 80);
     value store len x = do {
-      if len >= String.length buff.val then
-        buff.val := buff.val ^ string_create (String.length buff.val)
+      if len >= string_length buff.val then
+        buff.val :=
+          string_cat buff.val (string_create (string_length buff.val))
       else ();
       string_set buff.val len x;
       succ len
@@ -1458,7 +1496,7 @@ module Buff =
         if i == String.length s then len
         else add_rec (store len s.[i]) (succ i)
     ;
-    value get len = String.sub buff.val 0 len;
+    value get len = bytes_to_string (string_sub buff.val 0 len);
   end
 ;
 
