@@ -27,8 +27,6 @@ value string_of_string_token loc s =
   try Plexing.eval_string loc s with [ Failure _ as exn → Ploc.raise loc exn ]
 ;
 
-value glob_fname = ref "";
-
 value mkloc loc =
   let bp = Ploc.first_pos loc in
   let ep = Ploc.last_pos loc in
@@ -36,7 +34,8 @@ value mkloc loc =
   let bolp = Ploc.bol_pos loc in
   let lnuml = Ploc.line_nb_last loc in
   let bolpl = Ploc.bol_pos_last loc in
-  ocaml_location (glob_fname.val, lnum, bolp, lnuml, bolpl, bp, ep)
+  let fname = Ploc.file_name loc in
+  ocaml_location (fname, lnum, bolp, lnuml, bolpl, bp, ep)
 ;
 
 value mktyp ?{alg_attributes=[]} loc d = ocaml_mktyp ~{alg_attributes=alg_attributes} (mkloc loc) d;
@@ -999,9 +998,9 @@ and expr =
       | _ → error loc "bad left part of assignment"
       ]
   | ExAsr loc <:expr< False >> →
-      mkexp loc (ocaml_pexp_assertfalse glob_fname.val (mkloc loc))
+      mkexp loc (ocaml_pexp_assertfalse "-" (mkloc loc))
   | ExAsr loc e →
-      mkexp loc (ocaml_pexp_assert glob_fname.val (mkloc loc) (expr e))
+      mkexp loc (ocaml_pexp_assert "-" (mkloc loc) (expr e))
   | ExBae loc dotop e1 el →
       match (uv dotop, uv el) with [
         (".", el) -> expr (bigarray_get loc e1 el)
@@ -1167,6 +1166,7 @@ and expr =
         expr <:expr< $lid:dotop$ $e1$ $aexp$ >>
       ]
   | ExStr loc s →
+      let (_, s) = uv s in
       mkexp loc
         (Pexp_constant
            (mkconst loc (ocaml_pconst_string (string_of_string_token loc (uv s)) (mkloc loc) None)))
@@ -1374,8 +1374,8 @@ and sig_item s l =
   | SgTypExten loc te →
       [mksig loc (ocaml_psig_typext (type_extension loc te)) :: l]
   | SgUse loc fn sl →
-      Ploc.call_with glob_fname (uv fn)
-        (fun () → List.fold_right (fun (si, _) → sig_item si) (uv sl) l) ()
+      List.fold_right (fun (si, _) → sig_item si) (uv sl) l
+
   | SgVal loc n t attrs →
       let vn = uv n in
       [mksig loc (ocaml_psig_value vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn ([], t) [])) :: l]
@@ -1516,8 +1516,7 @@ and str_item s l =
   | StTypExten loc te →
       [mkstr loc (ocaml_pstr_typext (type_extension loc te)) :: l]
   | StUse loc fn sl →
-      Ploc.call_with glob_fname (uv fn)
-        (fun () → List.fold_right (fun (si, _) → str_item si) (uv sl) l) ()
+      List.fold_right (fun (si, _) → str_item si) (uv sl) l
   | StVal loc rf pel →
       [mkstr loc (Pstr_value (mkrf (uv rf)) (List.map mkpe (uv pel))) :: l]
   | StXtr loc _ _ → error loc "bad ast StXtr"
@@ -1797,12 +1796,10 @@ and extension (idloc, payload) =
 ;
 
 value interf fname ast = do {
-  glob_fname.val := fname;
   List.fold_right sig_item ast []
 };
 
 value implem fname ast = do {
-  glob_fname.val := fname;
   List.fold_right str_item ast []
 };
 
@@ -1819,7 +1816,6 @@ value directive loc =
       [ Some pdir_bool → pdir_bool False
       | None → error loc "no such kind of directive in this ocaml version" ]
   | <:expr< $longid:li$ >> -> Pdir_ident (longid_long_id li)
-  | <:expr< $longid:li$ >> -> Pdir_ident (longid_long_id li)
   | <:expr< $longid:li$ . $lid:id$ >> -> Pdir_ident (longid_lident_long_id (Some <:vala< li >>,<:vala< id >>))
   | <:expr<  $lid:id$ >> -> Pdir_ident (Lident id)
   | e →
@@ -1845,7 +1841,6 @@ value phrase =
   [ StDir loc d dp →
       ocaml_ptop_dir (mkloc loc) (uv d) (directive_args loc (uv dp))
   | si → do {
-      glob_fname.val := Plexing.input_file.val;
       Ptop_def (str_item si [])
     } ]
 ;
