@@ -78,6 +78,7 @@ and patt =
   | PaArr of loc and V (list patt)
   | PaChr of loc and V string
   | PaExc of loc and patt
+  | PaEff of loc and patt and patt
   | PaFlo of loc and V string
   | PaInt of loc and V string and string
   | PaLab of loc and V (list (patt * V (option patt)))
@@ -132,7 +133,7 @@ and expr =
   | ExSeq of loc and V (list expr)
   | ExSnd of loc and expr and V string
   | ExSte of loc and V string and expr and V (list expr)
-  | ExStr of loc and V string
+  | ExStr of loc and V (loc * V string)
   | ExTry of loc and expr and V (list case_branch)
   | ExTup of loc and V (list expr)
   | ExTyc of loc and expr and ctyp
