@@ -7,7 +7,7 @@
 
 open Printf;
 
-value version = "8.03.01";
+value version = "8.03.06";
 value syntax_name = ref "";
 
 value ocaml_version =
@@ -80,6 +80,10 @@ value alg_attribute = Grammar.Entry.create gram "alg_attribute";
 value alg_attributes = Grammar.Entry.create gram "alg_attributes";
 value ext_attributes = Grammar.Entry.create gram "ext_attributes";
 
+value stream_expr = Grammar.Entry.create gram "stream_expr";
+value stream_parser = Grammar.Entry.create gram "stream_parser";
+value stream_match = Grammar.Entry.create gram "stream_match";
+
 type ast_transducer_t 'a = {
     name : string
   ; parse : ref (option (Stream.t char -> 'a))
@@ -473,7 +477,8 @@ value undef x = ref (fun _ -> failwith x);
 value print_interf = undef "no printer";
 value print_implem = undef "no printer";
 
-value pr_expr = Eprinter.make ~{fail=Pp_debug.Pp_MLast.show_expr} "expr";
+value show_expr e = Format.asprintf "%a" Pp_debug.Pp_MLast.pp_expr e ;
+value pr_expr = Eprinter.make ~{fail=show_expr} "expr";
 value pr_patt = Eprinter.make "patt";
 value pr_ctyp = Eprinter.make "type";
 value pr_str_item = Eprinter.make "str_item";
