@@ -614,6 +614,26 @@ value check_type_binder =
 ;
 
 
+IFNDEF STRICT THEN
+value watch_str_expr (x : string) (e : MLast.expr) = () ;
+value watch__str_expr (x : string) (e : MLast.expr) = () ;
+value watch_locstr_expr (x : (Ploc.t * string)) (e : MLast.expr) = () ;
+value watch__locstr_expr (x : (Ploc.t * string)) (e : MLast.expr) = () ;
+ELSE
+value watch_str_expr (x : string) (e : MLast.expr) = (); 
+value watch__str_expr (x : Ploc.vala string) (e : MLast.expr) = (); 
+value watch_locstr_expr (x : (Ploc.t * Ploc.vala string)) (e : MLast.expr) = () ;
+value watch__locstr_expr (x : Ploc.vala (Ploc.t * Ploc.vala string)) (e : MLast.expr) = () ;
+END;
+
+value pa_fails : Stream.t 'a -> string = parser [ ] ;
+
+value fails =
+  Grammar.Entry.of_parser gram "fails"
+    pa_fails
+;
+
+
 (* -- begin copy from pa_r to q_MLast -- *)
 
 EXTEND
@@ -627,6 +647,17 @@ EXTEND
     check_type_binder
     ext_attributes
     ;
+  located_rawstring: [ [
+      s = V RAWSTRING ->
+      let (delimsize,s) = Asttools.split_rawstring (Pcaml.unvala s) in
+      let loc = Asttools.narrow_loc loc (delimsize+2) (delimsize+2) in
+      (loc, <:vala< s >>)
+    ] ] ;
+  located_string: [ [
+      s = V STRING ->
+      let loc = Asttools.narrow_loc loc 1 1 in
+      (loc, s)
+    ] ] ;
   attribute_id:
   [ [ l = LIST1 [ i = LIDENT -> i | i = UIDENT -> i ] SEP "." -> (loc, String.concat "." l)
     | s = STRING -> (loc, s)
@@ -1071,7 +1102,11 @@ EXTEND
       | s = V INT_L → <:expr< $_int64:s$ >>
       | s = V INT_n → <:expr< $_nativeint:s$ >>
       | s = V FLOAT → <:expr< $_flo:s$ >>
-      | s = V STRING → <:expr< $_str:s$ >>
+
+      | s = V fails "str" -> <:expr< $_str:s$ >>
+      | s = located_string → <:expr< $locstr:s$ >>
+      | s = V located_rawstring "locstr" → <:expr< $_locstr:s$ >>
+
       | s = V CHAR → <:expr< $_chr:s$ >>
       | "." -> <:expr< . >>
       | e = alg_extension -> <:expr< [% $_extension:e$ ] >>
@@ -1229,6 +1264,9 @@ EXTEND
       | s = V INT_n → <:patt< $_nativeint:s$ >>
       | s = V FLOAT → <:patt< $_flo:s$ >>
       | s = V STRING → <:patt< $_str:s$ >>
+      | s = RAWSTRING →
+        let (_,s) = Asttools.split_rawstring s in
+        <:patt< $str:s$ >>
       | s = V CHAR → <:patt< $_chr:s$ >>
       | "+"; s = V INT → <:patt< $_int:s$ >>
       | "+"; s = V FLOAT → <:patt< $_flo:s$ >>
@@ -1865,19 +1903,20 @@ EXTEND
   ;
   top_phrase:
     [ [ ph = phrase → Some ph
-      | ";" -> Some <:str_item< declare end >>
+      | [";"|";;"] -> Some <:str_item< declare end >>
       | EOI → None ] ]
   ;
   use_file:
-    [ [ "#"; n = LIDENT; dp = OPT expr; ";" →
+    [ [ "#"; n = LIDENT; dp = OPT expr; [";"|";;"] →
           ([<:str_item< # $lid:n$ $opt:dp$ >>], True)
       | si = str_item; ";"; (sil, stopped) = SELF → ([si :: sil], stopped)
+      | si = str_item; ";;" → ([si], False)
       | EOI → ([], False) ] ]
   ;
   phrase:
-    [ [ "#"; n = LIDENT; dp = OPT expr; ";" →
+    [ [ "#"; n = LIDENT; dp = OPT expr; [";"|";;"] →
           <:str_item< # $lid:n$ $opt:dp$ >>
-      | sti = str_item; ";" → sti ] ]
+      | sti = str_item; [";"|";;"] → sti ] ]
   ;
   expr: LEVEL "simple"
     [ [ x = QUOTATION →
