@@ -1036,7 +1036,21 @@ EXTEND
       | s = SV INT_L → Qast.Node "ExInt" [Qast.Loc; s; Qast.Str "L"]
       | s = SV INT_n → Qast.Node "ExInt" [Qast.Loc; s; Qast.Str "n"]
       | s = SV FLOAT → Qast.Node "ExFlo" [Qast.Loc; s]
-      | s = SV STRING → Qast.Node "ExStr" [Qast.Loc; s]
+      | s = SV STRING → do {
+        Qast.Node "ExStr" [Qast.Loc; Qast.VaVal (Qast.Tuple [Qast.Loc; s])]
+        }
+      | s = SV RAWSTRING "locstr" -> do {
+          match s with [
+              Qast.VaVal (Str s) ->
+              let (_,s) = Asttools.split_rawstring s in
+              Qast.Node "ExStr" [Qast.Loc; Qast.VaVal (Qast.Tuple [Qast.Loc; Qast.VaVal (Qast.Str s)])]
+            | VaVal (VaAnt  _ _ _) ->
+              Qast.Node "ExStr" [Qast.Loc; s]
+            | (VaAnt  _ _ _) ->
+              Qast.Node "ExStr" [Qast.Loc; s]
+            | _ -> Ploc.raise loc (Failure "Internal error: a locstr should have been either a raw string or an antiquotation")
+            ]
+        }
       | s = SV CHAR → Qast.Node "ExChr" [Qast.Loc; s]
       | e = alg_extension -> Qast.Node "ExExten" [Qast.Loc; e]
       | i = SV LIDENT → Qast.Node "ExLid" [Qast.Loc; i]
