@@ -1,33 +1,43 @@
 (* camlp5r *)
 (* exparser.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (* #load "q_MLast.cmo" *)
 
 type spat_comp =
-    SpTrm of MLast.loc * MLast.patt * MLast.expr option MLast.v
-  | SpNtr of MLast.loc * MLast.patt * MLast.expr
-  | SpLet of MLast.loc * MLast.patt * MLast.expr
-  | SpLhd of MLast.loc * MLast.patt list list
-  | SpStr of MLast.loc * MLast.patt
+  Exparser_types.spat_comp =
+      SpTrm of MLast.loc * MLast.patt * MLast.expr option MLast.v
+    | SpNtr of MLast.loc * MLast.patt * MLast.expr
+    | SpLet of MLast.loc * MLast.patt * MLast.expr
+    | SpLhd of MLast.loc * MLast.patt list list * MLast.expr option MLast.v
+    | SpStr of MLast.loc * MLast.patt
 ;;
 type sexp_comp =
-    SeTrm of MLast.loc * MLast.expr
-  | SeNtr of MLast.loc * MLast.expr
+  Exparser_types.sexp_comp =
+      SeTrm of MLast.loc * MLast.expr
+    | SeNtr of MLast.loc * MLast.expr
 ;;
 
 type spat_comp_opt =
-    SpoNoth
-  | SpoBang
-  | SpoQues of MLast.expr
+  Exparser_types.spat_comp_opt =
+      SpoNoth
+    | SpoBang
+    | SpoQues of MLast.expr
+;;
+
+type spat_parser_ast =
+  MLast.patt option *
+    ((spat_comp * spat_comp_opt) list * MLast.patt option * MLast.expr) list
 ;;
 
 let strm_n = "strm__";;
 let peek_fun loc =
-  MLast.ExAcc (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "peek"))
+  MLast.ExFle
+    (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")), (None, "peek"))
 ;;
 let junk_fun loc =
-  MLast.ExAcc (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "junk"))
+  MLast.ExFle
+    (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")), (None, "junk"))
 ;;
 
 (* Parsers *)
@@ -35,7 +45,9 @@ let junk_fun loc =
 let rec pattern_eq_expression p e =
   match p, e with
     MLast.PaLid (_, a), MLast.ExLid (_, b) -> a = b
-  | MLast.PaUid (_, a), MLast.ExUid (_, b) -> a = b
+  | MLast.PaLong (_, MLast.LiUid (_, a), []),
+    MLast.ExLong (_, MLast.LiUid (_, b)) ->
+      a = b
   | MLast.PaApp (_, p1, p2), MLast.ExApp (_, e1, e2) ->
       pattern_eq_expression p1 e1 && pattern_eq_expression p2 e2
   | _ -> false
@@ -51,8 +63,8 @@ let is_raise_failure e =
   match e with
     MLast.ExApp
       (_, MLast.ExLid (_, "raise"),
-       MLast.ExAcc
-         (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure"))) ->
+       MLast.ExLong
+         (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"))) ->
       true
   | _ -> false
 ;;
@@ -61,8 +73,8 @@ let rec handle_failure e =
   match e with
     MLast.ExTry
       (_, te,
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None, e]) ->
       handle_failure e
   | MLast.ExMat (_, me, pel) ->
@@ -73,16 +85,17 @@ let rec handle_failure e =
          | _ -> false)
         pel
   | MLast.ExLet (_, false, pel, e) ->
-      List.for_all (fun (p, e) -> handle_failure e) pel && handle_failure e
+      List.for_all (fun (p, e, _) -> handle_failure e) pel && handle_failure e
   | MLast.ExSeq (_, el) -> List.for_all handle_failure el
-  | MLast.ExAcc (_, MLast.ExUid (_, _), _) | MLast.ExLid (_, _) |
-    MLast.ExInt (_, _, "") | MLast.ExStr (_, _) | MLast.ExChr (_, _) |
-    MLast.ExFun (_, _) | MLast.ExUid (_, _) ->
+  | MLast.ExFle (_, MLast.ExLong (_, MLast.LiUid (_, _)), (None, _)) |
+    MLast.ExLid (_, _) | MLast.ExInt (_, _, "") | MLast.ExStr (_, (_, _)) |
+    MLast.ExChr (_, _) | MLast.ExFun (_, _) |
+    MLast.ExLong (_, MLast.LiUid (_, _)) ->
       true
   | MLast.ExApp (_, MLast.ExLid (_, "raise"), e) ->
       begin match e with
-        MLast.ExAcc
-          (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure")) ->
+        MLast.ExLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure")) ->
           false
       | _ -> true
       end
@@ -91,10 +104,12 @@ let rec handle_failure e =
   | _ -> false
 and no_raising_failure_fun =
   function
-    MLast.ExUid (_, _) -> true
+    MLast.ExLong (_, MLast.LiUid (_, _)) -> true
   | MLast.ExLid (_, _) -> false
-  | MLast.ExAcc (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "peek")) |
-    MLast.ExAcc (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "junk")) ->
+  | MLast.ExFle
+      (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")), (None, "peek")) |
+    MLast.ExFle
+      (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")), (None, "junk")) ->
       true
   | MLast.ExApp (_, x, y) -> no_raising_failure_fun x && handle_failure y
   | _ -> false
@@ -104,19 +119,19 @@ let rec subst v e =
   match e with
     MLast.ExLid (loc, x) ->
       let x = if x = v then strm_n else x in MLast.ExLid (loc, x)
-  | MLast.ExUid (_, _) -> e
+  | MLast.ExLong (_, MLast.LiUid (_, _)) -> e
   | MLast.ExInt (_, _, "") -> e
   | MLast.ExChr (_, _) -> e
-  | MLast.ExStr (_, _) -> e
-  | MLast.ExAcc (_, _, _) -> e
+  | MLast.ExStr (_, (_, _)) -> e
+  | MLast.ExFle (_, _, (None, _)) -> e
   | MLast.ExLet (loc, rf, pel, e) ->
       MLast.ExLet (loc, rf, List.map (subst_pe v) pel, subst v e)
   | MLast.ExApp (loc, e1, e2) -> MLast.ExApp (loc, subst v e1, subst v e2)
   | MLast.ExTup (loc, el) -> MLast.ExTup (loc, List.map (subst v) el)
   | _ -> raise Not_found
-and subst_pe v (p, e) =
+and subst_pe v (p, e, attrs) =
   match p with
-    MLast.PaLid (_, v') when v <> v' -> p, subst v e
+    MLast.PaLid (_, v') when v <> v' -> p, subst v e, attrs
   | _ -> raise Not_found
 ;;
 
@@ -126,7 +141,7 @@ Pcaml.add_option "-no-pa-opt" (Arg.Clear optim) "No parsers optimization.";;
 let rec perhaps_bound s =
   function
     MLast.ExTup (_, el) -> List.exists (perhaps_bound s) el
-  | MLast.ExUid (_, _) | MLast.ExStr (_, _) -> false
+  | MLast.ExLong (_, MLast.LiUid (_, _)) | MLast.ExStr (_, (_, _)) -> false
   | _ -> true
 ;;
 
@@ -141,7 +156,9 @@ let stream_pattern_component skont ckont =
     SpTrm (loc, p, wo) ->
       MLast.ExMat
         (loc, MLast.ExApp (loc, peek_fun loc, MLast.ExLid (loc, strm_n)),
-         [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p), wo,
+         [MLast.PaApp
+            (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p),
+          wo,
           MLast.ExSeq
             (loc,
              [MLast.ExApp (loc, junk_fun loc, MLast.ExLid (loc, strm_n));
@@ -155,9 +172,7 @@ let stream_pattern_component skont ckont =
              [MLast.PaTyc
                 (_, MLast.PaLid (_, v),
                  MLast.TyApp
-                   (_,
-                    MLast.TyAcc
-                      (_, MLast.TyUid (_, "Stream"), MLast.TyLid (_, "t")),
+                   (_, MLast.TyAcc (_, MLast.LiUid (_, "Stream"), "t"),
                     MLast.TyAny _)),
               None, e])
           when v = strm_n ->
@@ -171,66 +186,87 @@ let stream_pattern_component skont ckont =
           else
             MLast.ExTry
               (loc, e,
-               [MLast.PaAcc
-                  (loc, MLast.PaUid (loc, "Stream"),
-                   MLast.PaUid (loc, "Failure")),
+               [MLast.PaLong
+                  (loc,
+                   MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                   []),
                 None, ckont])
         else if is_raise_failure ckont then
           let p = wildcard_if_not_bound p skont in
-          MLast.ExLet (loc, false, [p, e], skont)
+          MLast.ExLet (loc, false, [p, e, []], skont)
         else if is_raise ckont then
           let tst =
             if handle_failure e then e
             else
               MLast.ExTry
                 (loc, e,
-                 [MLast.PaAcc
-                    (loc, MLast.PaUid (loc, "Stream"),
-                     MLast.PaUid (loc, "Failure")),
+                 [MLast.PaLong
+                    (loc,
+                     MLast.LiAcc
+                       (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                     []),
                   None, ckont])
           in
           let p = wildcard_if_not_bound p skont in
-          MLast.ExLet (loc, false, [p, tst], skont)
+          MLast.ExLet (loc, false, [p, tst, []], skont)
         else if
           pattern_eq_expression
-            (MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p)) skont
+            (MLast.PaApp
+               (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p))
+            skont
         then
           MLast.ExTry
-            (loc, MLast.ExApp (loc, MLast.ExUid (loc, "Some"), e),
-             [MLast.PaAcc
-                (loc, MLast.PaUid (loc, "Stream"),
-                 MLast.PaUid (loc, "Failure")),
+            (loc,
+             MLast.ExApp
+               (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Some")), e),
+             [MLast.PaLong
+                (loc,
+                 MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                 []),
               None, ckont])
         else
           MLast.ExMat
             (loc,
              MLast.ExTry
-               (loc, MLast.ExApp (loc, MLast.ExUid (loc, "Some"), e),
-                [MLast.PaAcc
-                   (loc, MLast.PaUid (loc, "Stream"),
-                    MLast.PaUid (loc, "Failure")),
-                 None, MLast.ExUid (loc, "None")]),
-             [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p), None, skont;
+               (loc,
+                MLast.ExApp
+                  (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Some")), e),
+                [MLast.PaLong
+                   (loc,
+                    MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                    []),
+                 None, MLast.ExLong (loc, MLast.LiUid (loc, "None"))]),
+             [MLast.PaApp
+                (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p),
+              None, skont;
               MLast.PaAny loc, None, ckont])
       else
         MLast.ExMat
           (loc,
            MLast.ExTry
-             (loc, MLast.ExApp (loc, MLast.ExUid (loc, "Some"), e),
-              [MLast.PaAcc
-                 (loc, MLast.PaUid (loc, "Stream"),
-                  MLast.PaUid (loc, "Failure")),
-               None, MLast.ExUid (loc, "None")]),
-           [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p), None, skont;
+             (loc,
+              MLast.ExApp
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Some")), e),
+              [MLast.PaLong
+                 (loc,
+                  MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                  []),
+               None, MLast.ExLong (loc, MLast.LiUid (loc, "None"))]),
+           [MLast.PaApp
+              (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p),
+            None, skont;
             MLast.PaAny loc, None, ckont])
   | SpLet (_, _, _) -> assert false
-  | SpLhd (loc, pl :: pll) ->
+  | SpLhd (loc, pl :: pll, wo) ->
       let mklistpat loc pl =
         List.fold_right
           (fun p1 p2 ->
              MLast.PaApp
-               (loc, MLast.PaApp (loc, MLast.PaUid (loc, "::"), p1), p2))
-          pl (MLast.PaUid (loc, "[]"))
+               (loc,
+                MLast.PaApp
+                  (loc, MLast.PaLong (loc, MLast.LiUid (loc, "::"), []), p1),
+                p2))
+          pl (MLast.PaLong (loc, MLast.LiUid (loc, "[]"), []))
       in
       let len = List.length pl in
       if List.exists (fun pl -> List.length pl <> len) pll then
@@ -248,20 +284,23 @@ let stream_pattern_component skont ckont =
              (loc,
               MLast.ExApp
                 (loc,
-                 MLast.ExAcc
-                   (loc, MLast.ExUid (loc, "Stream"),
-                    MLast.ExLid (loc, "npeek")),
+                 MLast.ExFle
+                   (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                    (None, "npeek")),
                  MLast.ExInt (loc, string_of_int len, "")),
               MLast.ExLid (loc, "strm__")),
-           [p, None, skont; MLast.PaAny loc, None, ckont])
-  | SpLhd (loc, []) -> ckont
+           [p, wo, skont; MLast.PaAny loc, None, ckont])
+  | SpLhd (loc, [], wo) -> ckont
   | SpStr (loc, p) ->
       try
         match p with
-          MLast.PaLid (_, v) -> subst v skont
+          MLast.PaLid (_, v) ->
+            MLast.ExLet
+              (loc, false,
+               [MLast.PaLid (loc, v), MLast.ExLid (loc, "strm__"), []], skont)
         | _ -> raise Not_found
       with Not_found ->
-        MLast.ExLet (loc, false, [p, MLast.ExLid (loc, strm_n)], skont)
+        MLast.ExLet (loc, false, [p, MLast.ExLid (loc, strm_n), []], skont)
 ;;
 
 let rec stream_pattern loc epo e ekont =
@@ -274,16 +313,17 @@ let rec stream_pattern loc epo e ekont =
              [ep,
               MLast.ExApp
                 (loc,
-                 MLast.ExAcc
-                   (loc, MLast.ExUid (loc, "Stream"),
-                    MLast.ExLid (loc, "count")),
-                 MLast.ExLid (loc, strm_n))],
+                 MLast.ExFle
+                   (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                    (None, "count")),
+                 MLast.ExLid (loc, strm_n)),
+              []],
              e)
       | _ -> e
       end
   | (SpLet (loc, p1, e1), _) :: spcl ->
       let skont = stream_pattern loc epo e ekont spcl in
-      MLast.ExLet (loc, false, [p1, e1], skont)
+      MLast.ExLet (loc, false, [p1, e1, []], skont)
   | (spc, err) :: spcl ->
       let skont =
         let ekont =
@@ -293,25 +333,28 @@ let rec stream_pattern loc epo e ekont =
                 (loc, MLast.ExLid (loc, "raise"),
                  MLast.ExApp
                    (loc,
-                    MLast.ExAcc
-                      (loc, MLast.ExUid (loc, "Stream"),
-                       MLast.ExUid (loc, "Error")),
+                    MLast.ExLong
+                      (loc,
+                       MLast.LiAcc
+                         (loc, MLast.LiUid (loc, "Stream"), "Error")),
                     estr))
           | SpoBang ->
               MLast.ExApp
                 (loc, MLast.ExLid (loc, "raise"),
-                 MLast.ExAcc
-                   (loc, MLast.ExUid (loc, "Stream"),
-                    MLast.ExUid (loc, "Failure")))
+                 MLast.ExLong
+                   (loc,
+                    MLast.LiAcc
+                      (loc, MLast.LiUid (loc, "Stream"), "Failure")))
           | SpoNoth ->
               MLast.ExApp
                 (loc, MLast.ExLid (loc, "raise"),
                  MLast.ExApp
                    (loc,
-                    MLast.ExAcc
-                      (loc, MLast.ExUid (loc, "Stream"),
-                       MLast.ExUid (loc, "Error")),
-                    MLast.ExStr (loc, "")))
+                    MLast.ExLong
+                      (loc,
+                       MLast.LiAcc
+                         (loc, MLast.LiUid (loc, "Stream"), "Error")),
+                    MLast.ExStr (loc, (loc, ""))))
         in
         stream_pattern loc epo e ekont spcl
       in
@@ -322,7 +365,10 @@ let stream_patterns_term loc ekont tspel =
   let pel =
     List.map
       (fun (p, w, loc, spcl, epo, e) ->
-         let p = MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p) in
+         let p =
+           MLast.PaApp
+             (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p)
+         in
          let e =
            let ekont =
              function
@@ -331,25 +377,28 @@ let stream_patterns_term loc ekont tspel =
                    (loc, MLast.ExLid (loc, "raise"),
                     MLast.ExApp
                       (loc,
-                       MLast.ExAcc
-                         (loc, MLast.ExUid (loc, "Stream"),
-                          MLast.ExUid (loc, "Error")),
+                       MLast.ExLong
+                         (loc,
+                          MLast.LiAcc
+                            (loc, MLast.LiUid (loc, "Stream"), "Error")),
                        estr))
              | SpoBang ->
                  MLast.ExApp
                    (loc, MLast.ExLid (loc, "raise"),
-                    MLast.ExAcc
-                      (loc, MLast.ExUid (loc, "Stream"),
-                       MLast.ExUid (loc, "Failure")))
+                    MLast.ExLong
+                      (loc,
+                       MLast.LiAcc
+                         (loc, MLast.LiUid (loc, "Stream"), "Failure")))
              | SpoNoth ->
                  MLast.ExApp
                    (loc, MLast.ExLid (loc, "raise"),
                     MLast.ExApp
                       (loc,
-                       MLast.ExAcc
-                         (loc, MLast.ExUid (loc, "Stream"),
-                          MLast.ExUid (loc, "Error")),
-                       MLast.ExStr (loc, "")))
+                       MLast.ExLong
+                         (loc,
+                          MLast.LiAcc
+                            (loc, MLast.LiUid (loc, "Stream"), "Error")),
+                       MLast.ExStr (loc, (loc, ""))))
            in
            let skont = stream_pattern loc epo e ekont spcl in
            MLast.ExSeq
@@ -378,8 +427,8 @@ let rec parser_cases loc =
     [] ->
       MLast.ExApp
         (loc, MLast.ExLid (loc, "raise"),
-         MLast.ExAcc
-           (loc, MLast.ExUid (loc, "Stream"), MLast.ExUid (loc, "Failure")))
+         MLast.ExLong
+           (loc, MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure")))
   | spel ->
       if !optim then
         match group_terms spel with
@@ -394,9 +443,9 @@ let rec parser_cases loc =
         | [] ->
             MLast.ExApp
               (loc, MLast.ExLid (loc, "raise"),
-               MLast.ExAcc
-                 (loc, MLast.ExUid (loc, "Stream"),
-                  MLast.ExUid (loc, "Failure")))
+               MLast.ExLong
+                 (loc,
+                  MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure")))
 ;;
 
 (* optim: left factorization of consecutive rules *)
@@ -470,9 +519,7 @@ let mk_rule x =
           MLast.PaTyc
             (loc, MLast.PaLid (loc, strm_n),
              MLast.TyApp
-               (loc,
-                MLast.TyAcc
-                  (loc, MLast.TyUid (loc, "Stream"), MLast.TyLid (loc, "t")),
+               (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                 MLast.TyAny loc))
         in
         MLast.ExFun (loc, [p, None, e])
@@ -502,7 +549,7 @@ let left_factorize rl =
 
 (* Converting into AST *)
 
-let cparser loc bpo pc =
+let cparser loc (bpo, pc) =
   let pc = left_factorize pc in
   let e = parser_cases loc pc in
   let e =
@@ -514,10 +561,11 @@ let cparser loc bpo pc =
            [bp,
             MLast.ExApp
               (loc,
-               MLast.ExAcc
-                 (loc, MLast.ExUid (loc, "Stream"),
-                  MLast.ExLid (loc, "count")),
-               MLast.ExLid (loc, strm_n))],
+               MLast.ExFle
+                 (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                  (None, "count")),
+               MLast.ExLid (loc, strm_n)),
+            []],
            e)
     | None -> e
   in
@@ -526,9 +574,7 @@ let cparser loc bpo pc =
     MLast.PaTyc
       (loc, MLast.PaLid (loc, strm_n),
        MLast.TyApp
-         (loc,
-          MLast.TyAcc
-            (loc, MLast.TyUid (loc, "Stream"), MLast.TyLid (loc, "t")),
+         (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
           MLast.TyAny loc))
   in
   MLast.ExFun (loc, [p, None, e])
@@ -536,16 +582,16 @@ let cparser loc bpo pc =
 
 let rec is_not_bound s =
   function
-    MLast.ExUid (_, _) -> true
+    MLast.ExLong (_, MLast.LiUid (_, _)) -> true
   | MLast.ExApp
       (_, MLast.ExLid (_, "raise"),
-       MLast.ExAcc
-         (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure"))) ->
+       MLast.ExLong
+         (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"))) ->
       true
   | _ -> false
 ;;
 
-let cparser_match loc me bpo pc =
+let cparser_match loc me (bpo, pc) =
   let pc = left_factorize pc in
   let iloc = Ploc.with_comment loc "" in
   let pc = parser_cases iloc pc in
@@ -558,10 +604,11 @@ let cparser_match loc me bpo pc =
            [bp,
             MLast.ExApp
               (loc,
-               MLast.ExAcc
-                 (loc, MLast.ExUid (loc, "Stream"),
-                  MLast.ExLid (loc, "count")),
-               MLast.ExLid (loc, strm_n))],
+               MLast.ExFle
+                 (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                  (None, "count")),
+               MLast.ExLid (loc, strm_n)),
+            []],
            pc)
     | None -> pc
   in
@@ -578,11 +625,9 @@ let cparser_match loc me bpo pc =
          [MLast.PaTyc
             (loc, p,
              MLast.TyApp
-               (loc,
-                MLast.TyAcc
-                  (loc, MLast.TyUid (loc, "Stream"), MLast.TyLid (loc, "t")),
+               (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                 MLast.TyAny loc)),
-          me],
+          me, []],
          e)
 ;;
 
@@ -590,14 +635,15 @@ let cparser_match loc me bpo pc =
 
 let rec not_computing =
   function
-    MLast.ExLid (_, _) | MLast.ExUid (_, _) | MLast.ExInt (_, _, "") |
-    MLast.ExFlo (_, _) | MLast.ExChr (_, _) | MLast.ExStr (_, _) ->
+    MLast.ExLid (_, _) | MLast.ExLong (_, MLast.LiUid (_, _)) |
+    MLast.ExInt (_, _, "") | MLast.ExFlo (_, _) | MLast.ExChr (_, _) |
+    MLast.ExStr (_, (_, _)) ->
       true
   | MLast.ExApp (_, x, y) -> is_cons_apply_not_computing x && not_computing y
   | _ -> false
 and is_cons_apply_not_computing =
   function
-    MLast.ExUid (_, _) -> true
+    MLast.ExLong (_, MLast.LiUid (_, _)) -> true
   | MLast.ExLid (_, _) -> false
   | MLast.ExApp (_, x, y) -> is_cons_apply_not_computing x && not_computing y
   | _ -> false
@@ -605,7 +651,7 @@ and is_cons_apply_not_computing =
 
 let slazy loc e =
   match e with
-    MLast.ExApp (_, f, MLast.ExUid (_, "()")) ->
+    MLast.ExApp (_, f, MLast.ExLong (_, MLast.LiUid (_, "()"))) ->
       begin match f with
         MLast.ExLid (_, _) -> f
       | _ -> MLast.ExFun (loc, [MLast.PaAny loc, None, e])
@@ -617,20 +663,23 @@ let rec cstream gloc =
   function
     [] ->
       let loc = gloc in
-      MLast.ExAcc
-        (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "sempty"))
+      MLast.ExFle
+        (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+         (None, "sempty"))
   | [SeTrm (loc, e)] ->
       if not_computing e then
         MLast.ExApp
           (loc,
-           MLast.ExAcc
-             (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "ising")),
+           MLast.ExFle
+             (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+              (None, "ising")),
            e)
       else
         MLast.ExApp
           (loc,
-           MLast.ExAcc
-             (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "lsing")),
+           MLast.ExFle
+             (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+              (None, "lsing")),
            slazy loc e)
   | SeTrm (loc, e) :: secl ->
       if not_computing e then
@@ -638,9 +687,9 @@ let rec cstream gloc =
           (loc,
            MLast.ExApp
              (loc,
-              MLast.ExAcc
-                (loc, MLast.ExUid (loc, "Stream"),
-                 MLast.ExLid (loc, "icons")),
+              MLast.ExFle
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                 (None, "icons")),
               e),
            cstream gloc secl)
       else
@@ -648,9 +697,9 @@ let rec cstream gloc =
           (loc,
            MLast.ExApp
              (loc,
-              MLast.ExAcc
-                (loc, MLast.ExUid (loc, "Stream"),
-                 MLast.ExLid (loc, "lcons")),
+              MLast.ExFle
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                 (None, "lcons")),
               slazy loc e),
            cstream gloc secl)
   | [SeNtr (loc, e)] ->
@@ -658,8 +707,9 @@ let rec cstream gloc =
       else
         MLast.ExApp
           (loc,
-           MLast.ExAcc
-             (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "slazy")),
+           MLast.ExFle
+             (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+              (None, "slazy")),
            slazy loc e)
   | SeNtr (loc, e) :: secl ->
       if not_computing e then
@@ -667,8 +717,9 @@ let rec cstream gloc =
           (loc,
            MLast.ExApp
              (loc,
-              MLast.ExAcc
-                (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "iapp")),
+              MLast.ExFle
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                 (None, "iapp")),
               e),
            cstream gloc secl)
       else
@@ -676,8 +727,9 @@ let rec cstream gloc =
           (loc,
            MLast.ExApp
              (loc,
-              MLast.ExAcc
-                (loc, MLast.ExUid (loc, "Stream"), MLast.ExLid (loc, "lapp")),
+              MLast.ExFle
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Stream")),
+                 (None, "lapp")),
               slazy loc e),
            cstream gloc secl)
 ;;
