@@ -7,7 +7,7 @@
 
 open Printf;;
 
-let version = "8.03.01";;
+let version = "8.03.06";;
 let syntax_name = ref "";;
 
 let ocaml_version =
@@ -85,6 +85,10 @@ let alg_attribute = Grammar.Entry.create gram "alg_attribute";;
 let alg_attributes = Grammar.Entry.create gram "alg_attributes";;
 let ext_attributes = Grammar.Entry.create gram "ext_attributes";;
 
+let stream_expr = Grammar.Entry.create gram "stream_expr";;
+let stream_parser = Grammar.Entry.create gram "stream_parser";;
+let stream_match = Grammar.Entry.create gram "stream_match";;
+
 type 'a ast_transducer_t =
   { name : string;
     parse : (char Stream.t -> 'a) option ref;
@@ -477,7 +481,8 @@ let undef x = ref (fun _ -> failwith x);;
 let print_interf = undef "no printer";;
 let print_implem = undef "no printer";;
 
-let pr_expr = Eprinter.make ~fail:Pp_debug.Pp_MLast.show_expr "expr";;
+let show_expr e = Format.asprintf "%a" Pp_debug.Pp_MLast.pp_expr e;;
+let pr_expr = Eprinter.make ~fail:show_expr "expr";;
 let pr_patt = Eprinter.make "patt";;
 let pr_ctyp = Eprinter.make "type";;
 let pr_str_item = Eprinter.make "str_item";;
