@@ -738,7 +738,7 @@ Toploop.print_out_type.val := format_to_format_doc__printer print_out_type;
   Toploop.print_out_module_type.val :=
   recover_from_print_failure ~{fname="print_out_module_type"} (format_to_format_doc__printer print_out_module_type) Toploop.print_out_module_type.val;
 Toploop.print_out_signature.val :=
-  recover_from_print_failure ~{fname="print_out_signature"} (format_to_format_doc__printer print_out_signature) (format_to_format_doc__printer Pp_outcometree.pp_out_sig_item_list);
+  recover_from_print_failure ~{fname="print_out_signature"} (format_to_format_doc__printer print_out_signature) (format_to_format_doc__printer (Fmt.list Pp_outcometree.pp_out_sig_item));
   Toploop.print_out_sig_item.val :=
   recover_from_print_failure ~{fname="print_out_sig_item"} (format_to_format_doc__printer print_out_sig_item) Toploop.print_out_sig_item.val;
 Toploop.print_out_phrase.val := print_out_phrase;
