@@ -106,7 +106,7 @@ let lexer_ = {Plexing.tok_func = lexer_ ;
  Plexing.tok_using = (fun _ -> ()); Plexing.tok_removing = (fun _ -> ());
  Plexing.tok_match = Plexing.default_match;
  Plexing.tok_text = Plexing.lexer_text;
- Plexing.tok_comm = None} ;;
+ Plexing.tok_comm = None ; Plexing.kwds = Hashtbl.create 23 } ;;
 
 let g = Grammar.gcreate lexer_ ;;
 let expr = Grammar.Entry.create g "expression";;
