@@ -111,7 +111,7 @@ value lexer_ = {Plexing.tok_func = lexer_;
  Plexing.tok_using _ = (); Plexing.tok_removing _ = ();
  Plexing.tok_match = Plexing.default_match;
  Plexing.tok_text = Plexing.lexer_text;
- Plexing.tok_comm = None} ;
+ Plexing.tok_comm = None ; Plexing.kwds = Hashtbl.create 23 } ;
 
 value g = Grammar.gcreate lexer_;
 value expr = Grammar.Entry.create g "expression";
